pragma Source_Reference (000001, "asECOAMain_MODIMPL.adb");
-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - Application Component Body
-- #
-- # $Id: asECOAMain_MODIMPL.adb,v 2.2 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################
-------------------------------------------------------------------
-- @file asecoamain_modimpl.adb
-- Module Interface package for Module asECOAMain_MODIMPL
-------------------------------------------------------------------

with ECOA, asECOAMain_MODIMPL_Container, Location, Text_IO;
 use ECOA, asECOAMain_MODIMPL_Container;
package body asECOAMain_MODIMPL is

   -- Runtime lifecycle API

   procedure INITIALIZE_received
      (Context : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   		posnNIL : Location.Location := (0, 0 );
   begin
   		Context.User_Context.acPosn  := posnNIL;
   		Context.User_Context.acAlt   := 0;
   		Context.User_Context.acSpeed := 0;
   end INITIALIZE_received;

   procedure START_received
      (Context : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end START_received;

   procedure STOP_received
      (Context : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end STOP_received;

   procedure SHUTDOWN_received
      (Context : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end SHUTDOWN_received;

	------------------------------------------------------------------------------------------------
	
	procedure computeSteer( Context : in out asECOAMain_MODIMPL_Container.Context_Type;
							acPosn : in Location.Location;
							acAlt : in ECOA.Signed_32_Type;
							acSpeed : in ECOA.Signed_32_Type;
							wp: in Location.Location;
							steer : out ECOA.Signed_32_Type ) is
		now : ECOA.HR_Time_Type;
	begin
		-- Compute a steer setting from where we are (acPosn/acAlt/asSpeed) to where we'd like to be (wp):
			-- a) compute direction of wp from acPosn
			-- b) determine the amount of steer necessary to head the a/c towards the wp
			-- c) determine if we need to change alt
			-- d) determine if we need to change speed in order to get there on time
			--    etc.
		steer := 0;
		--
		Get_Relative_Local_Time( Context, now );
		Text_IO.put( ECOA.Seconds_Type'image(now.Seconds) & ":" & ECOA.NanoSeconds_Type'image(now.Nanoseconds) & " computed...   " & ascii.cr );
		--
	end computeSteer;
		
	------------------------------------------------------------------------------------------------
	
   procedure tick_Received
      (Context : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
		errc : ECOA.Return_Status_Type;
		acPosn : Location.Location;
		acAlt, acSpeed, steer : ECOA.Signed_32_Type;
		nextWP : Location.Location;
   begin
		-- Get the inputs
		getPosn_Request_Sync( Context, acPosn, errc );
		getAlt_Request_Sync( Context, acAlt, errc );
		getSpeed_Request_Sync( Context, acSpeed, errc );
		getWayPoint_Request_Sync( Context, nextWP, errc );
		--
		-- Do computations
		computeSteer( Context, acPosn, acAlt, acSpeed, nextWP, steer );
		--
		-- Output results
		putSteer_Send( Context, steer );
		--
		-- Wait until the epoch
		--
		-- In ECOA, that simply means wait until the next tick of the TriggerInstance...
		--
   end tick_Received;

end asECOAMain_MODIMPL;
