pragma Source_Reference (000001, "asECOAIF_MODIMPL.adb");
-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - Interface Component Body
-- #
-- # $Id: asECOAIF_MODIMPL.adb,v 2.2 2017/11/06 15:33:37 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################
-------------------------------------------------------------------
-- @file asecoaif_modimpl.adb
-- Module Interface package for Module asECOAIF_MODIMPL
-------------------------------------------------------------------

with ECOA, asECOAIF_MODIMPL_Container, System, System.Storage_Elements, Location;
 use ECOA, asECOAIF_MODIMPL_Container, System.Storage_Elements;
package body asECOAIF_MODIMPL is

	function toLog( s : String ) return ECOA.Log_Type;
	
   -- Runtime lifecycle API

   procedure INITIALIZE_received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end INITIALIZE_received;

   procedure START_received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end START_received;

   procedure STOP_received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end STOP_received;

   procedure SHUTDOWN_received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type)
   is
   begin
      null; -- User Code Here
   end SHUTDOWN_received;

	------------------------------------------------------------------------------------------------

	-- ADC channel numbers
	ALTIM_CHAN : constant := 1;
	SPEED_CHAN : constant := 2;
	
	-- ADC hardware addresses
	ADCBase    : constant := 16#F000#;
	ADCAddr    : array( 1..4 ) of System.Address := ( System.Storage_Elements.To_Address(ADCBase + 16#10#),
													  System.Storage_Elements.To_Address(ADCBase + 16#20#),
													  System.Storage_Elements.To_Address(ADCBase + 16#30#),
													  System.Storage_Elements.To_Address(ADCBase + 16#40#) );
	-- Data bus port numbers
	POSN_PORT  : constant := 21;
	WP_PORT    : constant := 58;
	
	-- DAC hardware address
	DACAddr    : constant := 16#ABCD#;
	
	------------------------------------------------------------------------------------------------

	package lastGood is
		acAlt, acSpeed : ECOA.Signed_32_Type;
		acPosn         : Location.Location := ( 0, 0 );
		wp             : Location.Location := ( 0, 0 );
	end lastGood;
	
	------------------------------------------------------------------------------------------------

	-- ****************************************************************
	-- Comments marked "--@" are code that has specific dependencies
	-- not relevant, and therefore masked, for this demonstration...
	-- ****************************************************************

	-- Mimics reading a block of stuff from a serial / network channel... 
	procedure readBlocking( port : in ECOA.Signed_32_Type; msgBufferRef : in System.Address; bufLength : in ECOA.Signed_32_Type; timeOut : in duration; msgLength : out ECOA.Signed_32_Type; isOK : out ECOA.Boolean_8_Type ) is
	begin
		if timeout > 0.0 then
			-- Read data bus port or time out...
--@			DataBus.setTimeOut( port, timeout );
--@			msgLength := DataBus.Receive( busAddress, port, msgBufferRef, bufLength );
			null;
		else
			-- Read bus port if data available...
--@			if DataBus.hasData( port ) then
--@				msgLength := DataBus.Receive( busAddress, port, msgBufferRef, bufLength );
--@			else
				msgLength := 0;
--@			end if;
		end if;
		--
		if msgLength > 0 then
			isOK := true;
		else
			isOK := false;
		end if;
	end readBlocking;
	
	------------------------------------------------------------------------------------------------
	
	procedure readADC( chan : in ECOA.Signed_32_Type; result : out ECOA.Signed_32_Type; isOK : out ECOA.Boolean_8_Type ) is
		ADCInput : ECOA.Signed_32_Type := 0; 
--@			for ADCInput'Address use ADCAddr( chan );  -- don't try this at home children...!
		ADCIsReady : ECOA.Boolean_8_Type := true;
--@			for ADCIsReady'Address use System.Storage_Elements.To_Address(ADCBase);
	begin
		if ADCIsReady then
			-- read the ADC channel chan
			result := ADCInput;
		else
			-- return a dummy result
			result := 0;
		end if;
		isOK := ADCIsReady;
	end readADC;
	
	------------------------------------------------------------------------------------------------
	
   procedure getPosn_Request_Received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type;
       ID      : in     ECOA.Unsigned_32_Type)
   is
  		errc   : ECOA.Return_Status_Type;
   		msg    : ECOA.Log_Type;
		value  : Location.Location;
		msgLen : ECOA.Signed_32_Type;
		readOK : ECOA.Boolean_8_Type;
		acPosn : Location.Location;
	begin
		acPosn := ( 0, 0 );
		--
		readBlocking( POSN_PORT, value'address, value'size, 0.0, msgLen, readOK );
		if readOK then
			acPosn := value;
			lastGood.acPosn := value;
		else
--@			Log_Warning( Context, toLog( "error: read of bus port " & integer'image( POSN_PORT ) & " timedout" ));
--@			Log_Warning( Context, toLog( "using last good value for acPosn" ));
			acPosn := lastGood.acPosn;
		end if;
		getPosn_Response_Send( Context, ID, acPosn, errc );
   end getPosn_Request_Received;

	------------------------------------------------------------------------------------------------
	
   procedure getAlt_Request_Received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type;
       ID      : in     ECOA.Unsigned_32_Type)
   is
   		errc   : ECOA.Return_Status_Type;
   		msg    : ECOA.Log_Type;
		value  : ECOA.Signed_32_Type;
		readOK : ECOA.Boolean_8_Type;
		acAlt  : ECOA.Signed_32_Type;
   begin
		readADC( ALTIM_CHAN, value, readOK );
		if readOK then
			acAlt := value;
			lastGood.acAlt := value;
		else
--@			Log_Warning( Context, toLog( "error: ADC not ready" ));
--@			Log_Warning( Context, toLog( "using last good value for acAlt" ));
			acAlt := lastGood.acAlt;
		end if;
		getAlt_Response_Send( Context, ID, acAlt, errc );
   end getAlt_Request_Received;

	------------------------------------------------------------------------------------------------
	
   procedure getSpeed_Request_Received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type;
       ID      : in     ECOA.Unsigned_32_Type)
   is
   		errc    : ECOA.Return_Status_Type;
   		msg     : ECOA.Log_Type;
		value   : ECOA.Signed_32_Type;
		readOK  : ECOA.Boolean_8_Type;
		acSpeed : ECOA.Signed_32_Type;
   begin
		readADC( SPEED_CHAN, value, readOK );
		if readOK then
			acSpeed := value;
			lastGood.acSpeed := value;
		else
--@			Log_Warning( Context, toLog( "error: ADC not ready" ));
--@			Log_Warning( Context, toLog( "using last good value for acSpeed" ));
			acSpeed := lastGood.acSpeed;
		end if;
		getSpeed_Response_Send( Context, ID, acSpeed, errc );
   end getSpeed_Request_Received;

	------------------------------------------------------------------------------------------------
	
   procedure getWayPoint_Request_Received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type;
       ID      : in     ECOA.Unsigned_32_Type)
   is
   		errc   : ECOA.Return_Status_Type;
   		msg    : ECOA.Log_Type;
		value  : Location.Location;
		msgLen : ECOA.Signed_32_Type;
		readOK : ECOA.Boolean_8_Type;
		wp     : Location.Location;
   begin
		wp := ( 0, 0 );
		--
		readBlocking( WP_PORT, value'address, value'size, 0.0, msgLen, readOK );
		if readOK then
			wp := value;
			lastGood.wp := value;
		else
			wp := lastGood.wp;
		end if;
 		getWayPoint_Response_Send( Context, ID, wp, errc );
   end getWayPoint_Request_Received;

	------------------------------------------------------------------------------------------------
	
   procedure putSteer_Received
      (Context : in out asECOAIF_MODIMPL_Container.Context_Type;
       steer : in ECOA.Signed_32_Type)
   is
		DACOutput : integer;
--@			for DACOutput'Address use System.Storage_Elements.To_Address(DACAddr);  -- don't try this at home children...!
	begin
		-- Output the steer setting to the control surface hardware
		DACOutput := integer(steer);
   end putSteer_Received;

	------------------------------------------------------------------------------------------------
	
	function toLog( s : String ) return ECOA.Log_Type is
		msg : ECOA.Log_Type;
	begin
		msg.Current_Size := s'length;
		for i in 1..msg.Current_Size loop
			msg.Data( i ) := ECOA.Character_8_Type( s( integer(i)));
		end loop;
		return msg;
	end toLog;
	
end asECOAIF_MODIMPL;
