-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - Interface Component Container Body
-- #
-- # $Id: asECOAIF_MODIMPL_Container.adb,v 2.2 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################
-------------------------------------------------------------------
-- @file asECOAIF_MODIMPL_Container.adb
-- Container Interface package body for Module asECOAIF_MODIMPL
-------------------------------------------------------------------
with Text_IO, Calendar, asECOA_Platform;
 use Calendar;
package body asECOAIF_MODIMPL_Container is

   -- Logging and fault management services API

   procedure Log_Trace
      (Context : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Info );
   end Log_Trace;

   procedure Log_Debug
      (Context : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Debug );
   end Log_Debug;

   procedure Log_Info
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Info );
   end Log_Info;

   procedure Log_Warning
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Warning );
   end Log_Warning;

   procedure Raise_Error
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Error );
   end Raise_Error;

   procedure Raise_Fatal_Error
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Error );
		--
		-- and die
		
   end Raise_Fatal_Error;

   -- Time services API

   procedure Get_Relative_Local_Time
      (Context             : in out Context_Type;
       Relative_Local_Time :    out ECOA.HR_Time_Type)
   is
   begin
      Relative_Local_Time := asECOA_Platform.Clock;
   end Get_Relative_Local_Time;

   procedure Get_UTC_Time
      (Context  : in out Context_Type;
       UTC_Time :    out ECOA.Global_Time_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      UTC_Time := asECOA_Platform.Clock;
      Status   := ECOA.Return_Status_Type_OK;
   end Get_UTC_Time;

   procedure Get_Absolute_System_Time
      (Context              : in out Context_Type;
       Absolute_System_Time :    out ECOA.Global_Time_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      Absolute_System_Time := asECOA_Platform.Clock;
      Status   := ECOA.Return_Status_Type_OK;
   end Get_Absolute_System_Time;

   -- Time resolution services API

   procedure Get_Relative_Local_Time_Resolution
      (Context                        : in out Context_Type;
       Relative_Local_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      Relative_Local_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_Relative_Local_Time_Resolution;

   procedure Get_UTC_Time_Resolution
      (Context                : in out Context_Type;
       UTC_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      UTC_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_UTC_Time_Resolution;

   procedure Get_Absolute_System_Time_Resolution
      (Context                         : in out Context_Type;
       Absolute_System_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      Absolute_System_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_Absolute_System_Time_Resolution;

   procedure save_non_volatile_context
      (Context      : in out Context_Type)
   is
   begin
      null; -- Container Code Here
   end Save_Non_Volatile_Context;

   procedure getPosn_Response_Send
      (Context : in out Context_Type;
       ID      : in     ECOA.Unsigned_32_Type;
       acPosn : in Location.Location;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      asECOA_Platform.getPosn_Response_Post( ID, acPosn, Status );
   end getPosn_Response_Send;

   procedure getAlt_Response_Send
      (Context : in out Context_Type;
       ID      : in     ECOA.Unsigned_32_Type;
       acAlt : in ECOA.Signed_32_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      asECOA_Platform.getAlt_Response_Post( ID, acAlt, Status );
   end getAlt_Response_Send;

   procedure getSpeed_Response_Send
      (Context : in out Context_Type;
       ID      : in     ECOA.Unsigned_32_Type;
       acSpeed : in ECOA.Signed_32_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      asECOA_Platform.getSpeed_Response_Post( ID, acSpeed, Status );
   end getSpeed_Response_Send;

   procedure getWayPoint_Response_Send
      (Context : in out Context_Type;
       ID      : in     ECOA.Unsigned_32_Type;
       wp      : in     Location.Location;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      asECOA_Platform.getWayPoint_Response_Post( ID, wp, Status );
   end getWayPoint_Response_Send;

end asECOAIF_MODIMPL_Container;
