-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - ECOA Platform Body
-- #
-- # $Id: asECOA_Platform.adb,v 2.2 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################

with asECOAMain_MODIMPL, asECOAMain_MODIMPL_Container,
	 asECOAIF_MODIMPL,   asECOAIF_MODIMPL_Container,
	 ECOA_Platform,
	 ECOA, Location,
	 System, Ada.Real_Time, Text_IO, Interfaces.C;
	 
 use Ada.Real_Time, ECOA, ECOA_Platform;
 
package body asECOA_Platform is

	type TimeSpec is record
		tv_sec  : Interfaces.C.long;
		tv_nsec : Interfaces.C.long;
	end record;
	pragma pack( TimeSpec );
	
	CLOCK_REALTIME : constant := 1;
	
	asECOAMain_MODIMPL_Context  : asECOAMain_MODIMPL_Container.Context_Type;
	asECOAMain_MODIMPL_pContext : asECOAMain_MODIMPL_PlatformContext; 

	asECOAIF_MODIMPL_Context   : asECOAIF_MODIMPL_Container.Context_Type;
	asECOAIF_MODIMPL_pContext  : asECOAIF_MODIMPL_PlatformContext; 

	function clock_gettime( clkid : Interfaces.C.unsigned_long; time : System.Address ) return integer;
	pragma Import( C, clock_gettime, "clock_gettime" );
	
	procedure Log( msg : String; msgLen : ECOA.Log_Elements_Index_Type; level : LogLevel ) is
		ts : ECOA.HR_Time_Type := Clock;
	begin
		Text_IO.put_line( ECOA.Seconds_Type'image( ts.seconds ) & ":" & ECOA.Nanoseconds_Type'image( ts.nanoseconds ) &
							":" & LogLevel'image( level ) &
							":" & msg );
	end Log;
  
  	function Clock return ECOA.HR_Time_Type is
  		enow : ECOA.HR_Time_Type;
  		now  : TimeSpec;
  		res  : integer;
  	begin
  		res := clock_gettime( CLOCK_REALTIME, now'address );
  		enow.seconds     := ECOA.Seconds_Type( now.tv_sec );
  		enow.nanoseconds := ECOA.Nanoseconds_Type( now.tv_nsec );
  		return enow;
  	end Clock;
  	
	function Clock return ECOA.Global_Time_Type is
  		enow : ECOA.Global_Time_Type;
  		now  : TimeSpec;
  		res  : integer;
 	begin
  		res := clock_gettime( CLOCK_REALTIME, now'address );
  		enow.seconds     := ECOA.Seconds_Type( now.tv_sec );
  		enow.nanoseconds := ECOA.Nanoseconds_Type( now.tv_nsec );
  		return enow;
	end Clock;
  
   procedure Stop_ticker
      (Context        : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		pContext.tickerState := ECOA_Platform.Module_States_Type_READY;
   end Stop_ticker;

   procedure Start_ticker
      (Context        : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		pContext.tickerState := ECOA_Platform.Module_States_Type_RUNNING;
   end Start_ticker;

   procedure Initialize_ticker
      (Context        : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		pContext.tickerState := ECOA_Platform.Module_States_Type_READY;
   end Initialize_ticker;

   procedure Shutdown_ticker
      (Context        : in out asECOAMain_MODIMPL_Container.Context_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		pContext.tickerState := ECOA_Platform.Module_States_Type_IDLE;
   end Shutdown_ticker;

	procedure Main is
		--
		nextTime : Ada.Real_Time.Time := Ada.Real_Time.Clock;
		--
	begin
		-- Initialize the Modules
		asECOAMain_MODIMPL_Context.Platform_Hook := asECOAMain_MODIMPL_pContext'Address;
		asECOAMain_MODIMPL_pContext.moduleState  := ECOA_Platform.Module_States_Type_IDLE;
		asECOAMain_MODIMPL_pContext.tickerState  := ECOA_Platform.Module_States_Type_IDLE;
		--
		asECOAIF_MODIMPL_Context.Platform_Hook   := asECOAIF_MODIMPL_pContext'Address;
		asECOAIF_MODIMPL_pContext.moduleState    := ECOA_Platform.Module_States_Type_IDLE;
		--
		asECOAMain_MODIMPL.INITIALIZE_received( asECOAMain_MODIMPL_Context );
			asECOAMain_MODIMPL_pContext.moduleState := ECOA_Platform.Module_States_Type_READY;
		asECOAIF_MODIMPL.INITIALIZE_received( asECOAIF_MODIMPL_Context );
			asECOAIF_MODIMPL_pContext.moduleState   := ECOA_Platform.Module_States_Type_READY;
		Initialize_ticker( asECOAMain_MODIMPL_Context ); 
		--
		asECOAMain_MODIMPL.START_received( asECOAMain_MODIMPL_Context );
			asECOAMain_MODIMPL_pContext.moduleState := ECOA_Platform.Module_States_Type_RUNNING;
		asECOAIF_MODIMPL.START_received( asECOAIF_MODIMPL_Context );
			asECOAIF_MODIMPL_pContext.moduleState   := ECOA_Platform.Module_States_Type_RUNNING;
		Start_ticker( asECOAMain_MODIMPL_Context );
		--
		loop
			--
			if asECOAMain_MODIMPL_pContext.tickerState = ECOA_Platform.Module_States_Type_RUNNING then
				asECOAMain_MODIMPL.tick_Received( asECOAMain_MODIMPL_Context );
			end if; 
			--
			-- Wait until the epoch
			nextTime := nextTime + Ada.Real_Time.Milliseconds (50);
			delay until nextTime;
		end loop;
	end Main;
	
	--######################################################################################
	
	acPosn : Location.Location;
	acPosnID : ECOA.Unsigned_32_Type;
	acPosnStatus : ECOA.Return_Status_Type;
	
	procedure getPosn_Request_Send
      (ID : ECOA.Unsigned_32_Type ) is
    begin
   		asECOAIF_MODIMPL.getPosn_Request_Received( asECOA_Platform.asECOAIF_MODIMPL_Context, ID );
    end getPosn_Request_Send;
      
    procedure getPosn_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acPosn : in Location.Location;
		 Status : in ECOA.Return_Status_Type ) is
	begin
		asECOA_Platform.acPosn := acPosn;
		asECOA_Platform.acPosnStatus := Status;
		asECOA_Platform.acPosnID := ID;
	end getPosn_Response_Post;
	
    procedure getPosn_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acPosn : out Location.Location;
		 Status : out ECOA.Return_Status_Type ) is
	begin
		if ID = asECOA_Platform.acPosnID then
			acPosn := asECOA_Platform.acPosn;
			Status := asECOA_Platform.acPosnStatus;
		else
			Status := ECOA.Return_Status_Type_INVALID_HANDLE;
		end if;
	end getPosn_Response_Get;
	
	--######################################################################################
	
	acAlt : ECOA.Signed_32_Type;
	acAltID : ECOA.Unsigned_32_Type;
	acAltStatus : ECOA.Return_Status_Type;
	
	procedure getAlt_Request_Send
      (ID : ECOA.Unsigned_32_Type ) is
    begin
   		asECOAIF_MODIMPL.getAlt_Request_Received( asECOA_Platform.asECOAIF_MODIMPL_Context, ID );
    end getAlt_Request_Send;
      
    procedure getAlt_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acAlt : in ECOA.Signed_32_Type;
		 Status : in ECOA.Return_Status_Type ) is
	begin
		asECOA_Platform.acAlt := acAlt;
		asECOA_Platform.acAltStatus := Status;
		asECOA_Platform.acAltID := ID;
	end getAlt_Response_Post;
	
    procedure getAlt_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acAlt : out ECOA.Signed_32_Type;
		 Status : out ECOA.Return_Status_Type ) is
	begin
		if ID = asECOA_Platform.acAltID then
			acAlt := asECOA_Platform.acAlt;
			Status := asECOA_Platform.acAltStatus;
		else
			Status := ECOA.Return_Status_Type_INVALID_HANDLE;
		end if;
	end getAlt_Response_Get;
	
	--######################################################################################
	
	acSpeed : ECOA.Signed_32_Type;
	acSpeedID : ECOA.Unsigned_32_Type;
	acSpeedStatus : ECOA.Return_Status_Type;
	
	procedure getSpeed_Request_Send
      (ID : ECOA.Unsigned_32_Type ) is
    begin
   		asECOAIF_MODIMPL.getSpeed_Request_Received( asECOA_Platform.asECOAIF_MODIMPL_Context, ID );
    end getSpeed_Request_Send;
      
    procedure getSpeed_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acSpeed : in ECOA.Signed_32_Type;
		 Status : in ECOA.Return_Status_Type ) is
	begin
		asECOA_Platform.acSpeed := acSpeed;
		asECOA_Platform.acSpeedStatus := Status;
		asECOA_Platform.acSpeedID := ID;
	end getSpeed_Response_Post;
	
    procedure getSpeed_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acSpeed : out ECOA.Signed_32_Type;
		 Status : out ECOA.Return_Status_Type ) is
	begin
		if ID = asECOA_Platform.acSpeedID then
			acSpeed := asECOA_Platform.acSpeed;
			Status := asECOA_Platform.acSpeedStatus;
		else
			Status := ECOA.Return_Status_Type_INVALID_HANDLE;
		end if;
	end getSpeed_Response_Get;
	
	--######################################################################################
	
	wp : Location.Location;
	wpID : ECOA.Unsigned_32_Type;
	wpStatus : ECOA.Return_Status_Type;
	
	procedure getWayPoint_Request_Send
      (ID : ECOA.Unsigned_32_Type ) is
    begin
   		asECOAIF_MODIMPL.getWayPoint_Request_Received( asECOA_Platform.asECOAIF_MODIMPL_Context, ID );
    end getWayPoint_Request_Send;
      
    procedure getWayPoint_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 wp : in Location.Location;
		 Status : in ECOA.Return_Status_Type ) is
	begin
		asECOA_Platform.wp := wp;
		asECOA_Platform.wpStatus := Status;
		asECOA_Platform.wpID := ID;
	end getWayPoint_Response_Post;
	
    procedure getWayPoint_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 wp : out Location.Location;
		 Status : out ECOA.Return_Status_Type ) is
	begin
		if ID = asECOA_Platform.wpID then
			wp := asECOA_Platform.wp;
			Status := asECOA_Platform.wpStatus;
		else
			Status := ECOA.Return_Status_Type_INVALID_HANDLE;
		end if;
	end getWayPoint_Response_Get;
	
	--######################################################################################
	
	procedure putSteer_Send( steer : in ECOA.Signed_32_Type ) is
	begin
		asECOAIF_MODIMPL.putSteer_Received( asECOA_Platform.asECOAIF_MODIMPL_Context, steer );
	end putSteer_Send;

	--######################################################################################
	
end asECOA_Platform;
