-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - Application Component Container Body
-- #
-- # $Id: asECOAMain_MODIMPL_Container.adb,v 2.2 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################
-------------------------------------------------------------------
-- @file asECOAMain_MODIMPL_Container.adb
-- Container Interface package body for Module asECOAMain_MODIMPL
-------------------------------------------------------------------
with Text_IO, Calendar, ECOA, asECOA_Platform, asECOAMain_MODIMPL, ECOA_Platform;
 use Calendar, ECOA, ECOA_Platform;
package body asECOAMain_MODIMPL_Container is

   -- Logging and fault management services API

   procedure Log_Trace
      (Context : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Info );
   end Log_Trace;

   procedure Log_Debug
      (Context : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Debug );
   end Log_Debug;

   procedure Log_Info
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Info );
   end Log_Info;

   procedure Log_Warning
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Warning );
   end Log_Warning;

   procedure Raise_Error
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Error );
   end Raise_Error;

   procedure Raise_Fatal_Error
      (Context  : in out Context_Type;
       Log      : in ECOA.Log_Type)
   is
   		msg : String( 1..integer(Log.current_size));
   begin
   		for i in 1..integer(Log.current_size) loop
   			msg(i) := Character( Log.Data( ECOA.Log_Elements_Index_Type(i)));
   		end loop;
		asECOA_Platform.Log( msg, Log.current_size, asECOA_Platform.Error );
		--
		-- and die
		
   end Raise_Fatal_Error;

	-----------------------------------------------------------------------------------------------------------
	
   -- Time services API

   procedure Get_Relative_Local_Time
      (Context             : in out Context_Type;
       Relative_Local_Time :    out ECOA.HR_Time_Type)
   is
   begin
      Relative_Local_Time := asECOA_Platform.Clock;
   end Get_Relative_Local_Time;

   procedure Get_UTC_Time
      (Context  : in out Context_Type;
       UTC_Time :    out ECOA.Global_Time_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      UTC_Time := asECOA_Platform.Clock;
      Status   := ECOA.Return_Status_Type_OK;
   end Get_UTC_Time;

   procedure Get_Absolute_System_Time
      (Context              : in out Context_Type;
       Absolute_System_Time :    out ECOA.Global_Time_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   begin
      Absolute_System_Time := asECOA_Platform.Clock;
      Status   := ECOA.Return_Status_Type_OK;
   end Get_Absolute_System_Time;

   -- Time resolution services API

   procedure Get_Relative_Local_Time_Resolution
      (Context                        : in out Context_Type;
       Relative_Local_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      Relative_Local_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_Relative_Local_Time_Resolution;

   procedure Get_UTC_Time_Resolution
      (Context                : in out Context_Type;
       UTC_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      UTC_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_UTC_Time_Resolution;

   procedure Get_Absolute_System_Time_Resolution
      (Context                         : in out Context_Type;
       Absolute_System_Time_Resolution :    out ECOA.Duration_Type)
   is
   begin
      Absolute_System_Time_Resolution := asECOA_Platform.ClockResolution;
   end Get_Absolute_System_Time_Resolution;

	-----------------------------------------------------------------------------------------------------------
	
   procedure save_non_volatile_context
      (Context      : in out Context_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
      null; -- Container Code Here
   end Save_Non_Volatile_Context;

	-----------------------------------------------------------------------------------------------------------
	
	getPosn_ID     : ECOA.Unsigned_32_Type := 0;
	
   procedure getPosn_Request_Sync
      (Context : in out Context_Type;
       acPosn : out Location.Location;
       Status  :    out ECOA.Return_Status_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		getPosn_ID := getPosn_ID + 1;
   		asECOA_Platform.getPosn_Request_Send( getPosn_ID );
   		asECOA_Platform.getPosn_Response_Get( getPosn_ID, acPosn, Status );
   end getPosn_Request_Sync;

	-----------------------------------------------------------------------------------------------------------
	
	getAlt_ID      : ECOA.Unsigned_32_Type := 0;

   procedure getAlt_Request_Sync
      (Context : in out Context_Type;
       acAlt : out ECOA.Signed_32_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		getAlt_ID := getAlt_ID + 1;
   		asECOA_Platform.getAlt_Request_Send( getAlt_ID );
   		asECOA_Platform.getAlt_Response_Get( getAlt_ID, acAlt, Status );
   end getAlt_Request_Sync;

	-----------------------------------------------------------------------------------------------------------
	
	getSpeed_ID    : ECOA.Unsigned_32_Type := 0;

   procedure getSpeed_Request_Sync
      (Context : in out Context_Type;
       acSpeed : out ECOA.Signed_32_Type;
       Status  :    out ECOA.Return_Status_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		getSpeed_ID := getSpeed_ID + 1;
   		asECOA_Platform.getSpeed_Request_Send( getSpeed_ID );
   		asECOA_Platform.getSpeed_Response_Get( getSpeed_ID, acSpeed, Status );
   end getSpeed_Request_Sync;

	-----------------------------------------------------------------------------------------------------------
	
	getWayPoint_ID : ECOA.Unsigned_32_Type := 0;

   procedure getWayPoint_Request_Sync
      (Context : in out Context_Type;
       wp : out Location.Location;
       Status  :    out ECOA.Return_Status_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		getWayPoint_ID := getWayPoint_ID + 1;
   		asECOA_Platform.getWayPoint_Request_Send( getWayPoint_ID );
   		asECOA_Platform.getWayPoint_Response_Get( getWayPoint_ID, wp, Status );
   end getWayPoint_Request_Sync;

	-----------------------------------------------------------------------------------------------------------
	
   procedure putSteer_Send
      (Context : in out Context_Type;
       steer : in ECOA.Signed_32_Type)
   is
   		pContext : asECOA_Platform.asECOAMain_MODIMPL_PlatformContext;
   			for pContext'address use Context.Platform_Hook;
   begin
   		asECOA_Platform.putSteer_Send( steer );
   end putSteer_Send;

end asECOAMain_MODIMPL_Container;
