-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "as ECOA" Ada Implementation - ECOA Platform Spec.
-- #
-- # $Id: asECOA_Platform.ads,v 2.2 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################

with ECOA, ECOA_Platform, Location;
package asECOA_Platform is

	type  asECOAMain_MODIMPL_PlatformContext is record
		moduleState : ECOA_Platform.Module_States_Type;
		tickerState : ECOA_Platform.Module_States_Type;
	end record;

	type  asECOAIF_MODIMPL_PlatformContext is record
		moduleState : ECOA_Platform.Module_States_Type;
		IOServiceAvailable : ECOA.Boolean_8_Type;
	end record;
	
	procedure Main;
	
	type LogLevel is ( Info, Trace, Debug, Warning, Error );
	
	procedure Log( msg : String; msgLen : ECOA.Log_Elements_Index_Type; level : LogLevel );
	
	ClockResolution : constant ECOA.Duration_Type := ( 0, 10000000 );

	function Clock return ECOA.HR_Time_Type;
	function Clock return ECOA.Global_Time_Type;
	
	--######################################################################################
	
	procedure getPosn_Request_Send
      	(ID : ECOA.Unsigned_32_Type );
    procedure getPosn_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acPosn : in Location.Location;
		 Status : in ECOA.Return_Status_Type );
	procedure getPosn_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acPosn : out Location.Location;
		 Status : out ECOA.Return_Status_Type );
		 
	--######################################################################################
	
	procedure getAlt_Request_Send
      	(ID : ECOA.Unsigned_32_Type );
    procedure getAlt_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acAlt : in ECOA.Signed_32_Type;
		 Status : in ECOA.Return_Status_Type );
	procedure getAlt_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acAlt : out ECOA.Signed_32_Type;
		 Status : out ECOA.Return_Status_Type );
		 
	--######################################################################################
	
	procedure getSpeed_Request_Send
      	(ID : ECOA.Unsigned_32_Type );
    procedure getSpeed_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 acSpeed : in ECOA.Signed_32_Type;
		 Status : in ECOA.Return_Status_Type );
	procedure getSpeed_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 acSpeed : out ECOA.Signed_32_Type;
		 Status : out ECOA.Return_Status_Type );
		 
	--######################################################################################
	
	procedure getWayPoint_Request_Send
      	(ID : ECOA.Unsigned_32_Type );
    procedure getWayPoint_Response_Post
    	(ID : ECOA.Unsigned_32_Type;
		 wp : in Location.Location;
		 Status : in ECOA.Return_Status_Type );
	procedure getWayPoint_Response_Get
    	(ID : ECOA.Unsigned_32_Type;
		 wp : out Location.Location;
		 Status : out ECOA.Return_Status_Type );
		 
	--######################################################################################
	
	procedure putSteer_Send( steer : in ECOA.Signed_32_Type );
	
	--######################################################################################
end asECOA_Platform;
