-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "Traditional" Ada Implementation - Application Body
-- #
-- # $Id: Trad.adb,v 2.1 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################

-- Something pretending to be a real-time, high-integrity, program

with Location, Ada.Real_Time, Text_IO, TradIF;
 use Location, Ada.Real_Time;
procedure Trad is

	------------------------------------------------------------------------------------------------
	
	procedure computeSteer( acPosn : in Location.Location; acAlt : in integer; acSpeed : in integer; wp: in Location.Location; steer : out integer ) is
		now : Ada.Real_Time.Time := Clock;
		sc : Ada.Real_Time.Seconds_Count;
		ts : Ada.Real_Time.Time_Span;
		ns : constant Ada.Real_Time.Time_Span := Ada.Real_Time.nanoseconds(1);
	begin
		-- Compute a steer setting from where we are (acPosn/acAlt/asSpeed) to where we'd like to be (wp):
			-- a) compute direction of wp from acPosn
			-- b) determine the amount of steer necessary to head the a/c towards the wp
			-- c) determine if we need to change alt
			-- d) determine if we need to change speed in order to get there on time
			--    etc.
			--
		steer := 0;
		--
		Ada.Real_Time.split( now, sc, ts );
		Text_IO.put( Ada.Real_Time.Seconds_Count'image(sc) & ":" & integer'image(ts/ns) & " computed...    " & ascii.cr );
		--
	end computeSteer;
		
	------------------------------------------------------------------------------------------------
	
	nextTime : Ada.Real_Time.Time := Ada.Real_Time.Clock;
	acPosn : Location.Location;
	acAlt, acSpeed, steer : integer;
	nextWP : Location.Location;
	
	------------------------------------------------------------------------------------------------

begin
	loop
		-- Get the inputs
		TradIF.getPosn( acPosn );
		TradIF.getAlt( acAlt );
		TradIF.getSpeed( acSpeed );
		TradIF.getWayPoint( nextWP );
		--
		-- Do computations
		computeSteer( acPosn, acAlt, acSpeed, nextWP, steer );
		--
		-- Output results
		TradIF.putSteer( steer );
		--
		-- Wait until the epoch
		nextTime := nextTime + Ada.Real_Time.Milliseconds (50);
		delay until nextTime;
	end loop;
end Trad;
